/*
 * imshift.c -- Shift an image within a fits  2D file
 *
 * John Kielkopf
 * kielkopf@louisville.edu
 * 
 *
 * Current Version 1.0 -- June 14, 2007
 *
 *   June 14, 2007 : Version 1.0 : First version from imaverage
 *   
 */

#include <string.h>
#include <stdio.h>
#include <math.h> 
#include "fitsio.h"

int main(int argc, char *argv[])
{
  fitsfile *afptr, *outfptr;  
  int status = 0;  
  int atype, anaxis, i, j, k;
  long npixels = 1, mpixels = 1, firstpix[2] = {1,1};
  long anaxes[2] = {1,1};
  int x,y,xdel,ydel;
  double *apix, *outpix;
  char *testdel;
  
  if (argc < 5) { 
    printf("Usage: imshift input_image x_shift y_shift output_image \n");
    printf("\n");
    printf("Shift an image within a fits file \n");
    printf("\n");
    printf("Example: \n");
    printf("  imshift image.fit 4 -5 newimage.fit\n");
    return(0);
  }

  xdel = strtod(argv[2],&testdel);
  ydel = strtod(argv[3],&testdel);
  if(testdel == argv[2])
  {
    printf("Shift for x is incorrect\n");
    return(0);
  }
  ydel = strtod(argv[3],&testdel); 
  if(testdel == argv[3])
  {
    printf("Shift for y is incorrect\n");
    return(0);
  }  
  
  /* Open an image */
    
  fits_open_file(&afptr, argv[1], READONLY, &status);

  /* Read dimensions */

  fits_get_img_dim(afptr, &anaxis, &status);
  fits_get_img_size(afptr, 2, anaxes, &status);
  
  /* Report errors if any */

  if (status) 
  {
    fits_report_error(stderr, status); 
    return(status);
  }

  /* Number of pixels to read in each row */
  
  mpixels = anaxes[0];
  
  /* Number of pixels in a column */
  
  npixels = anaxes[1];
  
  /* Allocate memory for one input row */ 

  apix = (double *) malloc(mpixels * sizeof(double)); 
  
  /* Allocate memory for entire output image */
  
  outpix = (double *) malloc(mpixels * npixels * sizeof(double)); 
  
  /* Initialize the final image memory to zero */
  
  for ( i = 0 ; i < ( mpixels * npixels ) ; i ++ )
  {
    outpix[i] = 0;
  }        
  
    
  printf("Image %d x %d will be shifted by (%d,%d)\n",mpixels,
    npixels,xdel,ydel);


  if ( (mpixels < xdel) || (npixels < ydel) )
  {
     printf(" Image shifts exceed image array size.\n");
     fits_close_file(afptr, &status); 
     return(0);          
  }
    

  /* Start with first pixel in each row */
  
  firstpix[0] = 1;
  
  /* Start with first pixel in sum array */
    
  j = 0;

  /* Process images one row at a time */
    
  for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
  {  
   
    /* Find starting pixel coordinate and number of pixels to read */
   
    if (fits_read_pix(afptr, TDOUBLE, firstpix, mpixels,
       NULL, apix, NULL, &status))
    break; 
    
    for (i = 0; i < anaxes[0]; i++) 
    {
      outpix[j] += apix[i];
      j++;          
    }
  }

  /* Create the new empty output file */

  fits_create_file(&outfptr, argv[4], &status);

  /* Copy all the header keywords from first input image */
  
  fits_copy_header(afptr, outfptr, &status);

  /* Create the shifted output image */
  
  j=0;
  k=0;
      
  for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
  {
    
    for(i=0; i < mpixels; i++)
    {
      apix[i]=0;
      x = i - xdel;
      y = j - ydel;
      if( ((x >= 0) && (x < mpixels)) && ((y >= 0) && (y < npixels)) )
      {
         k = (3056*y) + x;
         apix[i] = outpix[k];
      }      
    }
    j++;
    
    /* Write new values to output image */

    fits_write_pix(outfptr, TDOUBLE, firstpix, mpixels,
                 apix, &status); 
  }
  
  fits_close_file(afptr, &status);    
  fits_close_file(outfptr, &status);
  free(apix);
  free(outpix);
  
  /* Report any error message */
 
  if (status) fits_report_error(stderr, status); 
  return(status);
}

