/* read XEphem goto fifo and parse target coordinates */

void readXEphemTarget() 
{
  int i,j,k;
  double rahr,ramin,rasec;
  double decdeg,decmin,decsec;
  double tmpra,tmpdec,tmpepoch;
  double tmp_pm_ra,tmp_pm_dec;
  double tmpmag;
  double tmpdat;
  char fifostr[121];
  char rastr[121];
  char *raptr=rastr;  
  char inbuf[2];
  
/* A typical fixed target string is: */

/* Regulus,f|M|B7,10:08:22.3, 11:58:02,  1.35,2000,0 */
/* Name, type info, RA | pm, Dec | pm, Mag, Epoch, Identifier */

/* Return if the buffer is not open */

  if(fd_fifo_in<0)
    return;

/* Read the buffer */

  k=0;
  while(1) {
    i = read(fd_fifo_in,inbuf,1);
    if(i==-1) {
      fifostr[k]='\0';
      break;
    }
    if(k<120)
    fifostr[k++]=inbuf[0];
  }
  
/* Diagnostic of fifo string contents */

/*  printf("Fifo in: %s\n",fifostr); */
  
/* Examples from XEphem version 3.6.xx -- */

/* Arcturus,f|V|K1,14:15:39.7|-1093, 19:10:57|-1998,-0.04,2000,0  */
/* HD 124953,f|V|A8,14:16:04.2|43, 18:54:43|-28,5.98,2000,0       */
/* Crt Delta-12,f|S|G8,11:19:20.5|-122,-14:46:43|208,3.56,2000,0  */
/* GSC 0838-0788,f|S,10:32:32.4,  9:06:17,13.84,2000,0            */
/* M67,f|O|T2, 8:51:24.0, 11:49:00,6.90,2000,1500|1500|0          */
/* NGC 2539,f|O|T2, 8:10:36.9,-12:49:14,6.50,2000,900|900|0       */
/* Venus,P                                                        */

 printf("Fifo in: %s\n",fifostr); 
 
/* If the buffer was not empty then extract its contents */

  if (k>1) 
  {
    fifostr[k]='\0';

/* Remove any white space */

    i=0;
    j=0;
    while (i<=k)
    {
      if( fifostr[i]!=' ' )
      {
        rastr[j]=fifostr[i];
        j++;
      }
      i++;
    }  
    
/* Diagnostic of clean coordinate string contents */

/*  printf("Coordinate str: %s\n",rastr); */
      
    printf("Coordinate str: %s\n",rastr);

/* Skip the name and description fields by searching for delimiting commas */
/* This will skip orbital objects for which XEphem does not pass RA and Dec */   

    raptr=rastr;
    i=0;
    while(i<2)  
    {
      raptr=strchr(raptr,',');
      if(raptr==NULL) break;
      raptr++;
      i++;
    }
    
    
/* Parsing will have proceeded to the RA field if this is a catalog object */

    if(i==2)
    { 

/* Scan and save RA data */ 
/* Note that the XEphem proper motion is in milliarcseconds per year */ 
/* Our pm_ra is in seconds of time per year */

      tmpdat=0.;
      j=sscanf(raptr,"%lf:%lf:%lf|%lf", 
        &rahr,&ramin,&rasec,&tmpdat);

      if(j>0) 
      {
        tmpra = rahr + ramin/60. + rasec/3600.;
        tmp_pm_ra=tmpdat/15000.;
      }
       
/* Try to advance pointer to the Dec field */

      raptr=strchr(raptr,',');
      if(raptr != NULL)
      {
       raptr++;
       i++;
      }
    }
     

    if(i==3)
    {  

/* Now pointing at the Dec field so scan and save Dec data */
/* Note that the XEphem proper motion is in milliarcseconds per year */ 
/* Our pm_dec is in seconds of arc per year */

      tmpdat=0.;
      j=sscanf(raptr,"%lf:%lf:%lf|%lf", 
        &decdeg,&decmin,&decsec,&tmpdat);

      if(j>0) 
      {
        if(decdeg>0.)
        {
          tmpdec = decdeg + decmin/60. + decsec/3600.;
        }
        else
        { 
          tmpdec = decdeg - decmin/60. - decsec/3600.;
        }
        tmp_pm_dec=tmpdat/1000.;
      }     

/* Try to advance pointer to the Magnitude field */

      raptr=strchr(raptr,',');
      if(raptr != NULL)
      {
       raptr++;
       i++;
      }
            
    }

    if(i==4)
    {  

/* Now pointing at the Magnitude field so scan and save magnitude data */ 

      j=sscanf(raptr,"%lf", &tmpdat);

      if(j>0) 
      {
        tmpmag=tmpdat;
      }
    
/* Try to advance pointer to the Epoch field */

      raptr=strchr(raptr,',');
      if(raptr != NULL)
      {
       raptr++;
       i++;
      }   
    
    }
  
    if(i==5)
    {  

      /* Now pointing at the Epoch field so scan and show Epoch data */ 

      j=sscanf(raptr,"%lf", &tmpepoch);

      if(j>0) 
      {

         /* The catalog entry epoch has been found */
         /* Apply proper motion to EOD */

         ProperMotion(tmpepoch, &tmpra, &tmpdec, tmp_pm_ra, tmp_pm_dec);        
      
         /* Find the apparent coordinates including */
         /*   precession, nutation, and stellar aberration for the EOD */

         if ((tmpepoch > 1999.99) && (tmpepoch < 2000.01)) 
         {
           
           /* Epoch 2000.0 coordinates apply */
           /* Find the apparent position of the object for EOD */
           
           targetra = tmpra;
           targetdec = tmpdec;
           Apparent(&targetra, &targetdec, 1);

         }
         else 
         {
         
           /* Some other epoch applies.  Caution!!  */
           /* Set the epoch to 2000.0 and then proceed as before. */
           /* If coordinates are not FK5 this will not be accurate. */
                     
           PrecessToEOD(tmpepoch, &tmpra, &tmpdec);
           PrecessToEpoch(2000.0, &tmpra, &tmpdec);
          
           /* Epoch 2000.0 coordinates now apply */
           /* Find the apparent position of the object for EOD */

           targetra = tmpra;
           targetdec = tmpdec;
           Apparent(&targetra, &targetdec, 1);
         }
                           
         /* Show the entry at the epoch selected for display */         
         
         showTarget();
      }
      else
      {
         
         /* No epoch was found so assume EOD coordinates */
         
         targetra = tmpra;
         targetdec = tmpdec;       
         
         /* Show the entry at the epoch selected for display */
         
         showTarget();  
      }     
    }  
  }
}
